import requests

from twitchdl import CLIENT_ID
from twitchdl.parse import parse_playlists, parse_playlist


def authenticated_get(url, params={}):
    headers = {'Client-ID': CLIENT_ID}

    response = requests.get(url, params, headers=headers)
    response.raise_for_status()

    return response


def get_video(video_id):
    """
    https://dev.twitch.tv/docs/v5/reference/videos#get-video
    """
    url = "https://api.twitch.tv/kraken/videos/%d" % video_id

    return authenticated_get(url).json()


def get_channel_videos(channel_name, limit=20):
    """
    https://dev.twitch.tv/docs/v5/reference/channels#get-channel-videos
    """
    url = "https://api.twitch.tv/kraken/channels/%s/videos" % channel_name

    return authenticated_get(url, {
        "broadcast_type": "archive",
        "limit": limit,
    }).json()


def get_access_token(video_id):
    url = "https://api.twitch.tv/api/vods/%d/access_token" % video_id

    return authenticated_get(url).json()


def get_playlists(video_id, access_token):
    url = "http://usher.twitch.tv/vod/{}".format(video_id)

    response = requests.get(url, params={
        "nauth": access_token['token'],
        "nauthsig": access_token['sig'],
        "allow_source": "true",
        "player": "twitchweb",
    })
    response.raise_for_status()

    data = response.content.decode('utf-8')

    return parse_playlists(data)


def get_playlist_urls(url, start, end):
    response = requests.get(url)
    response.raise_for_status()

    data = response.content.decode('utf-8')
    return parse_playlist(url, data, start, end)



import re

from collections import OrderedDict
from datetime import timedelta
from twitchdl.exceptions import ConsoleError


def parse_playlists(data):
    media_pattern = re.compile(r'^#EXT-X-MEDIA:TYPE=VIDEO,GROUP-ID="(?P<group>\w+)",NAME="(?P<name>\w+)"')

    playlists = OrderedDict()
    n = 1
    name = None
    for line in data.split():
        match = re.match(media_pattern, line)
        if match:
            name = match.group('name')
        elif line.startswith('http'):
            playlists[n] = (name, line)
            n += 1

    return playlists


def _get_files(playlist, start, end):
    matches = re.findall(r"#EXTINF:(\d+)(\.\d+)?,.*?\s+(\d+.ts)", playlist)
    vod_start = 0
    for m in matches:
        filename = m[2]
        vod_duration = int(m[0])
        vod_end = vod_start + vod_duration

        # `vod_end > start` is used here becuase it's better to download a bit
        # more than a bit less, similar for the end condition
        start_condition = not start or vod_end > start
        end_condition = not end or vod_start < end

        if start_condition and end_condition:
            yield filename

        vod_start = vod_end


def parse_playlist(url, playlist, start, end):
    base_url = re.sub("/[^/]+$", "/{}", url)

    match = re.search(r"#EXT-X-TWITCH-TOTAL-SECS:(\d+)(.\d+)?", playlist)
    total_seconds = int(match.group(1))

    # Now that video duration is known, validate start and end max values
    if start and start > total_seconds:
        raise ConsoleError("Start time {} greater than video duration {}".format(
            timedelta(seconds=start),
            timedelta(seconds=total_seconds)
        ))

    if end and end > total_seconds:
        raise ConsoleError("End time {} greater than video duration {}".format(
            timedelta(seconds=end),
            timedelta(seconds=total_seconds)
        ))

    files = list(_get_files(playlist, start, end))
    return base_url, files
